@@ -43,7 +43,7 @@ def _calc_survfunc_right(time, status, weights=None, entry=None, compress=True,
     else:
         n = np.cumsum(n[::-1])[::-1]
 
-    # Only retain times where an event occured.
+    # Only retain times where an event occurred.
     if compress:
         ii = np.flatnonzero(d > 0)
         d = d[ii]
@@ -171,22 +171,22 @@ class CumIncidenceRight(object):
 
     Parameters
     ----------
-    time : array-like
+    time : array_like
         An array of times (censoring times or event times)
-    status : array-like
-        If status >= 1 indicates which event occured at time t.  If
+    status : array_like
+        If status >= 1 indicates which event occurred at time t.  If
         status = 0, the subject was censored at time t.
-    title : string
+    title : str
         Optional title used for plots and summary output.
-    freq_weights : array-like
+    freq_weights : array_like
         Optional frequency weights
-    exog : array-like
+    exog : array_like
         Optional, if present used to account for violation of
         independent censoring.
     bw_factor : float
         Band-width multiplier for kernel-based estimation.  Only
         used if exog is provided.
-    dimred : boolean
+    dimred : bool
         If True, proportional hazards regression models are used to
         reduce exog to two columns by predicting overall events and
         censoring in two separate models.  If False, exog is used
@@ -195,7 +195,7 @@ class CumIncidenceRight(object):
 
     Attributes
     ----------
-    times : array-like
+    times : array_like
         The distinct times at which the incidence rates are estimated
     cinc : list of arrays
         cinc[k-1] contains the estimated cumulative incidence rates
@@ -229,7 +229,7 @@ class CumIncidenceRight(object):
     D. Zeng (2004).  Estimating marginal survival function by
     adjusting for dependent censoring using many covariates.  Annals
     of Statistics 32:4.
-    http://arxiv.org/pdf/math/0409180.pdf
+    https://arxiv.org/pdf/math/0409180.pdf
     """
 
     def __init__(self, time, status, title=None, freq_weights=None,
@@ -271,22 +271,22 @@ class SurvfuncRight(object):
 
     Parameters
     ----------
-    time : array-like
+    time : array_like
         An array of times (censoring times or event times)
-    status : array-like
+    status : array_like
         Status at the event time, status==1 is the 'event'
         (e.g. death, failure), meaning that the event
         occurs at the given value in `time`; status==0
-        indicates that censoring has occured, meaning that
+        indicates that censoring has occurred, meaning that
         the event occurs after the given value in `time`.
-    entry : array-like, optional An array of entry times for handling
+    entry : array_like, optional An array of entry times for handling
         left truncation (the subject is not in the risk set on or
         before the entry time)
-    title : string
+    title : str
         Optional title used for plots and summary output.
-    freq_weights : array-like
+    freq_weights : array_like
         Optional frequency weights
-    exog : array-like
+    exog : array_like
         Optional, if present used to account for violation of
         independent censoring.
     bw_factor : float
@@ -295,18 +295,18 @@ class SurvfuncRight(object):
 
     Attributes
     ----------
-    surv_prob : array-like
+    surv_prob : array_like
         The estimated value of the survivor function at each time
         point in `surv_times`.
-    surv_prob_se : array-like
+    surv_prob_se : array_like
         The standard errors for the values in `surv_prob`.  Not available
         if exog is provided.
-    surv_times : array-like
+    surv_times : array_like
         The points where the survival function changes.
-    n_risk : array-like
+    n_risk : array_like
         The number of subjects at risk just before each time value in
         `surv_times`.  Not available if exog is provided.
-    n_events : array-like
+    n_events : array_like
         The number of events (e.g. deaths) that occur at each point
         in `surv_times`.  Not available if exog is provided.
 
@@ -325,7 +325,7 @@ class SurvfuncRight(object):
     D. Zeng (2004).  Estimating marginal survival function by
     adjusting for dependent censoring using many covariates.  Annals
     of Statistics 32:4.
-    http://arxiv.org/pdf/math/0409180.pdf
+    https://arxiv.org/pdf/math/0409180.pdf
     """
 
     def __init__(self, time, status, entry=None, title=None,
@@ -381,7 +381,7 @@ class SurvfuncRight(object):
         >>> li[0].set_color('purple')
         >>> li[1].set_color('purple')
 
-        Don't show the censoring points:
+        Do not show the censoring points:
 
         >>> fig = sf.plot()
         >>> ax = fig.get_axes()[0]
@@ -424,7 +424,7 @@ class SurvfuncRight(object):
         alpha : float
             The confidence interval has nominal coverage probability
             1 - `alpha`.
-        method : string
+        method : str
             Function to use for g-transformation, must be ...
 
         Returns
@@ -488,7 +488,7 @@ class SurvfuncRight(object):
         """
         Return a summary of the estimated survival function.
 
-        The summary is a datafram containing the unique event times,
+        The summary is a dataframe containing the unique event times,
         estimated survival function values, and related quantities.
         """
 
@@ -511,11 +511,11 @@ class SurvfuncRight(object):
             `1 - alpha` is the desired simultaneous coverage
             probability for the confidence region.  Currently alpha
             must be set to 0.05, giving 95% simultaneous intervals.
-        method : string
+        method : str
             The method used to produce the simultaneous confidence
             band.  Only the Hall-Wellner (hw) method is currently
             implemented.
-        transform : string
+        transform : str
             The used to produce the interval (note that the returned
             interval is on the survival probability scale regardless
             of which transform is used).  Only `log` and `arcsin` are
@@ -523,10 +523,10 @@ class SurvfuncRight(object):
 
         Returns
         -------
-        lcb : array-like
+        lcb : array_like
             The lower confidence limits corresponding to the points
             in `surv_times`.
-        ucb : array-like
+        ucb : array_like
             The upper confidence limits corresponding to the points
             in `surv_times`.
         """
@@ -570,15 +570,15 @@ def survdiff(time, status, group, weight_type=None, strata=None,
 
     Parameters
     ----------
-    time : array-like
+    time : array_like
         The event or censoring times.
-    status : array-like
+    status : array_like
         The censoring status variable, status=1 indicates that the
-        event occured, status=0 indicates that the observation was
+        event occurred, status=0 indicates that the observation was
         censored.
-    group : array-like
+    group : array_like
         Indicators of the two groups
-    weight_type : string
+    weight_type : str
         The following weight types are implemented:
             None (default) : logrank test
             fh : Fleming-Harrington, weights by S^(fh_p),
@@ -589,9 +589,9 @@ def survdiff(time, status, group, weight_type=None, strata=None,
             gb : Gehan-Breslow, weights by the number at risk
             tw : Tarone-Ware, weights by the square root of the number
                  at risk
-    strata : array-like
+    strata : array_like
         Optional stratum indicators for a stratified test
-    entry : array-like
+    entry : array_like
         Entry times to handle left truncation. The subject is not in
         the risk set on or before the entry time.
 
@@ -602,15 +602,11 @@ def survdiff(time, status, group, weight_type=None, strata=None,
     pvalue : The p-value for the chi^2 test
     """
 
-    # TODO: extend to handle more than two groups
-
     time = np.asarray(time)
     status = np.asarray(status)
     group = np.asarray(group)
 
     gr = np.unique(group)
-    if len(gr) != 2:
-        raise ValueError("logrank only supports two groups")
 
     if strata is None:
         obs, var = _survdiff(time, status, group, weight_type, gr,
@@ -627,11 +623,8 @@ def survdiff(time, status, group, weight_type=None, strata=None,
             obs += obs1
             var += var1
 
-    zstat = obs / np.sqrt(var)
-
-    # The chi^2 test statistic and p-value.
-    chisq = zstat**2
-    pvalue = 1 - chi2.cdf(chisq, 1)
+    chisq = obs.dot(np.linalg.solve(var, obs))  # (O - E).T * V^(-1) * (O - E)
+    pvalue = 1 - chi2.cdf(chisq, len(gr)-1)
 
     return chisq, pvalue
 
@@ -639,6 +632,8 @@ def survdiff(time, status, group, weight_type=None, strata=None,
 def _survdiff(time, status, group, weight_type, gr, entry=None,
               **kwargs):
     # logrank test for one stratum
+    # calculations based on https://web.stanford.edu/~lutian/coursepdf/unit6.pdf
+    # formula for variance better to take from https://web.stanford.edu/~lutian/coursepdf/survweek3.pdf
 
     # Get the unique times.
     if entry is None:
@@ -649,9 +644,9 @@ def _survdiff(time, status, group, weight_type, gr, entry=None,
         rtimes = rtimes[0:len(time)]
 
     # Split entry times by group if present (should use pandas groupby)
-    tse = [(gr[0], None), (gr[1], None)]
+    tse = [(gr_i, None) for gr_i in gr]
     if entry is not None:
-        for k in 0, 1:
+        for k, _ in enumerate(gr):
             ii = (group == gr[k])
             entry1 = entry[ii]
             tse[k] = (gr[k], entry1)
@@ -681,15 +676,7 @@ def _survdiff(time, status, group, weight_type, gr, entry=None,
 
     obs = sum(obsv)
     nrisk_tot = sum(nrisk)
-
-    # The variance of event counts in the first group.
-    r = nrisk[0] / np.clip(nrisk_tot, 1e-10, np.inf)
-    denom = nrisk_tot - 1
-    denom = np.clip(denom, 1e-10, np.inf)
-    var = obs * r * (1 - r) * (nrisk_tot - obs) / denom
-
-    # The expected number of events in the first group.
-    exp1 = obs * r
+    ix = np.flatnonzero(nrisk_tot > 1)
 
     weights = None
     if weight_type is not None:
@@ -715,17 +702,38 @@ def _survdiff(time, status, group, weight_type, gr, entry=None,
         else:
             raise ValueError("weight_type not implemented")
 
-    # The Z-scale test statistic (compare to normal reference
-    # distribution).
-    ix = np.flatnonzero(nrisk_tot > 1)
-    if weights is None:
-        obs = np.sum(obsv[0][ix] - exp1[ix])
-        var = np.sum(var[ix])
-    else:
-        obs = np.dot(weights[ix], obsv[0][ix] - exp1[ix])
-        var = np.dot(weights[ix]**2, var[ix])
+    dfs = len(gr) - 1
+    r = np.vstack(nrisk) / np.clip(nrisk_tot, 1e-10, np.inf)[None, :]  # each line is timeseries of r's. line per group
 
-    return obs, var
+    # The variance of event counts in each group.
+    groups_oe = []
+    groups_var = []
+
+    var_denom = nrisk_tot - 1
+    var_denom = np.clip(var_denom, 1e-10, np.inf)
+
+    # use the first group as a reference
+    for g in range(1, dfs+1):
+        # Difference between observed and  expected number of events in the group #g
+        oe = obsv[g] - r[g]*obs
+
+        # build one row of the dfs x dfs variance matrix
+        var_tensor_part = r[1:, :].T * (np.eye(1, dfs, g-1).ravel() - r[g, :, None])  # r*(1 - r) in multidim
+        var_scalar_part = obs * (nrisk_tot - obs) / var_denom
+        var = var_tensor_part * var_scalar_part[:, None]
+
+        if weights is not None:
+            oe = weights * oe
+            var = (weights**2)[:, None] * var
+
+        # sum over times and store
+        groups_oe.append(oe[ix].sum())
+        groups_var.append(var[ix].sum(axis=0))
+
+    obs_vec = np.hstack(groups_oe)
+    var_mat = np.vstack(groups_var)
+
+    return obs_vec, var_mat
 
 
 def plot_survfunc(survfuncs, ax=None):
@@ -734,7 +742,7 @@ def plot_survfunc(survfuncs, ax=None):
 
     Parameters
     ----------
-    survfuncs : object or array-like
+    survfuncs : object or array_like
         A single SurvfuncRight object, or a list or SurvfuncRight
         objects that are plotted together.
 
@@ -773,7 +781,7 @@ def plot_survfunc(survfuncs, ax=None):
     # If we have only a single survival function to plot, put it into
     # a list.
     try:
-        assert(type(survfuncs[0]) is SurvfuncRight)
+        assert type(survfuncs[0]) is SurvfuncRight
     except:
         survfuncs = [survfuncs]
 
@@ -798,7 +806,7 @@ def plot_survfunc(survfuncs, ax=None):
 
         # Plot the censored points.
         ii = np.flatnonzero(np.logical_not(sf.status))
-        ti = sf.time[ii]
+        ti = np.unique(sf.time[ii])
         jj = np.searchsorted(surv_times, ti) - 1
         sp = surv_prob[jj]
         ax.plot(ti, sp, '+', ms=12, color=li.get_color(),
