@@ -7,12 +7,12 @@ The one parameter exponential family distributions used by GLM.
 # for comparison to R, and McCullagh and Nelder
 
 
-import warnings
 import inspect
+
 import numpy as np
-from scipy import special
-from . import links as L
-from . import varfuncs as V
+from scipy import special, stats
+
+from . import links as L, varfuncs as V
 
 FLOAT_EPS = np.finfo(float).eps
 
@@ -76,13 +76,11 @@ class Family(object):
 
     def __init__(self, link, variance):
         if inspect.isclass(link):
-            warnmssg = "Calling Family(..) with a link class as argument "
-            warnmssg += "is deprecated.\n"
-            warnmssg += "Use an instance of a link class instead."
-            lvl = 2 if type(self) is Family else 3
-            warnings.warn(warnmssg,
-                          category=DeprecationWarning, stacklevel=lvl)
-            self.link = link()
+            warnmssg = (
+                "Calling Family(..) with a link class is not allowed. Use an "
+                "instance of a link class instead."
+            )
+            raise TypeError(warnmssg)
         else:
             self.link = link
         self.variance = variance
@@ -392,14 +390,14 @@ class Poisson(Family):
     statsmodels.genmod.families.family.Family : Parent class for all links.
     :ref:`links` : Further details on links.
     """
-    links = [L.log, L.identity, L.sqrt]
+    links = [L.Log, L.identity, L.sqrt]
     variance = V.mu
     valid = [0, np.inf]
     safe_links = [L.Log, ]
 
     def __init__(self, link=None):
         if link is None:
-            link = L.log()
+            link = L.Log()
         super(Poisson, self).__init__(link=link, variance=Poisson.variance)
 
     def _resid_dev(self, endog, mu):
@@ -493,6 +491,28 @@ class Poisson(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale=1., var_weights=1.):
+        r"""
+        Frozen Poisson distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is ignored.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+            var_weights are ignored for Poisson.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+
+        return stats.poisson(mu)
+
 
 class Gaussian(Family):
     """
@@ -519,7 +539,7 @@ class Gaussian(Family):
     :ref:`links` : Further details on links.
     """
 
-    links = [L.log, L.identity, L.inverse_power]
+    links = [L.Log, L.identity, L.inverse_power]
     variance = V.constant
     safe_links = links
 
@@ -545,7 +565,7 @@ class Gaussian(Family):
             Deviance residuals as defined below.
 
         Notes
-        --------
+        -----
         .. math::
 
            resid\_dev_i = (endog_i - \mu_i) ** 2
@@ -637,6 +657,28 @@ class Gaussian(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale, var_weights=1.):
+        r"""
+        Frozen Gaussian distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is required argument for get_distribution.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+
+        scale_n = scale / var_weights
+        return stats.norm(loc=mu, scale=np.sqrt(scale_n))
+
 
 class Gamma(Family):
     """
@@ -662,7 +704,7 @@ class Gamma(Family):
     statsmodels.genmod.families.family.Family : Parent class for all links.
     :ref:`links` : Further details on links.
     """
-    links = [L.log, L.identity, L.inverse_power]
+    links = [L.Log, L.identity, L.inverse_power]
     variance = V.mu_squared
     safe_links = [L.Log, ]
 
@@ -771,6 +813,30 @@ class Gamma(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale, var_weights=1.):
+        r"""
+        Frozen Gamma distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is required argument for get_distribution.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+        # combine var_weights with scale
+        scale_ = scale / var_weights
+        shape = 1 / scale_
+        scale_g = mu * scale_
+        return stats.gamma(shape, scale=scale_g)
+
 
 class Binomial(Family):
     """
@@ -780,7 +846,7 @@ class Binomial(Family):
     ----------
     link : a link instance, optional
         The default link for the Binomial family is the logit link.
-        Available links are logit, probit, cauchy, log, and cloglog.
+        Available links are logit, probit, cauchy, log, loglog, and cloglog.
         See statsmodels.genmod.families.links for more information.
 
     Attributes
@@ -809,7 +875,8 @@ class Binomial(Family):
     number of trials for each row.
     """
 
-    links = [L.logit, L.probit, L.cauchy, L.log, L.cloglog, L.identity]
+    links = [L.Logit, L.probit, L.cauchy, L.Log, L.CLogLog, L.LogLog,
+             L.identity]
     variance = V.binary  # this is not used below in an effort to include n
 
     # Other safe links, e.g. cloglog and probit are subclasses
@@ -817,7 +884,7 @@ class Binomial(Family):
 
     def __init__(self, link=None):  # , n=1.):
         if link is None:
-            link = L.logit()
+            link = L.Logit()
         # TODO: it *should* work for a constant n>1 actually, if freq_weights
         # is equal to n
         self.n = 1
@@ -942,10 +1009,10 @@ class Binomial(Family):
         y = endog * n  # Number of successes
 
         # note that mu is still in (0,1), i.e. not converted back
-        return (special.gammaln(n + 1) - special.gammaln(y + 1) -
-                special.gammaln(n - y + 1) +
-                y * np.log(mu / (1 - mu + 1e-20)) +
-                n * np.log(1 - mu + 1e-20)) * var_weights
+        return (
+            special.gammaln(n + 1) - special.gammaln(y + 1) -
+            special.gammaln(n - y + 1) + y * np.log(mu / (1 - mu + 1e-20)) +
+            n * np.log(1 - mu + 1e-20)) * var_weights
 
     def resid_anscombe(self, endog, mu, var_weights=1., scale=1.):
         r'''
@@ -1010,6 +1077,31 @@ class Binomial(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale=1., var_weights=1., n_trials=1):
+        r"""
+        Frozen Binomial distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is ignored.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+            var_weights are ignored for Poisson.
+        n_trials : int
+            Number of trials for the binomial distribution. The default is 1
+            which corresponds to a Bernoulli random variable.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+
+        return stats.binom(n=n_trials, p=mu)
+
 
 class InverseGaussian(Family):
     """
@@ -1042,7 +1134,7 @@ class InverseGaussian(Family):
     literature as the Wald distribution.
     """
 
-    links = [L.inverse_squared, L.inverse_power, L.identity, L.log]
+    links = [L.inverse_squared, L.inverse_power, L.identity, L.Log]
     variance = V.mu_cubed
     safe_links = [L.inverse_squared, L.Log, ]
 
@@ -1144,10 +1236,33 @@ class InverseGaussian(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale, var_weights=1.):
+        r"""
+        Frozen Inverse Gaussian distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is required argument for get_distribution.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+        # combine var_weights with scale
+        scale_ = scale / var_weights
+        mu_ig = mu * scale_
+        return stats.invgauss(mu_ig, scale=1 / scale_)
+
 
 class NegativeBinomial(Family):
     r"""
-    Negative Binomial exponential family.
+    Negative Binomial exponential family (corresponds to NB2).
 
     Parameters
     ----------
@@ -1187,7 +1302,7 @@ class NegativeBinomial(Family):
 
     with :math:`E[Y]=\mu\,` and :math:`Var[Y]=\mu+\alpha\mu^2`.
     """
-    links = [L.log, L.cloglog, L.identity, L.nbinom, L.Power]
+    links = [L.Log, L.CLogLog, L.identity, L.NegativeBinomial, L.Power]
     # TODO: add the ability to use the power links with an if test
     # similar to below
     variance = V.nbinom
@@ -1196,7 +1311,7 @@ class NegativeBinomial(Family):
     def __init__(self, link=None, alpha=1.):
         self.alpha = 1. * alpha  # make it at least float
         if link is None:
-            link = L.log()
+            link = L.Log()
         super(NegativeBinomial, self).__init__(
             link=link, variance=V.NegativeBinomial(alpha=self.alpha))
 
@@ -1330,6 +1445,29 @@ class NegativeBinomial(Family):
         resid *= np.sqrt(var_weights)
         return resid
 
+    def get_distribution(self, mu, scale=1., var_weights=1.):
+        r"""
+        Frozen NegativeBinomial distribution instance for given parameters
+
+        Parameters
+        ----------
+        mu : ndarray
+            Usually but not always the fitted mean response variable.
+        scale : float
+            The scale parameter is ignored.
+        var_weights : array_like
+            1d array of variance (analytic) weights. The default is 1.
+            var_weights are ignored for NegativeBinomial.
+
+        Returns
+        -------
+        distribution instance
+
+        """
+        size = 1. / self.alpha
+        prob = size / (size + mu)
+        return stats.nbinom(size, prob)
+
 
 class Tweedie(Family):
     """
@@ -1339,7 +1477,7 @@ class Tweedie(Family):
     ----------
     link : a link instance, optional
         The default link for the Tweedie family is the log link.
-        Available links are log and Power.
+        Available links are log, Power and any aliases of power.
         See statsmodels.genmod.families.links for more information.
     var_power : float, optional
         The variance power. The default is 1.
@@ -1356,7 +1494,7 @@ class Tweedie(Family):
         ``variance`` is an instance of
         statsmodels.genmod.families.varfuncs.Power
     Tweedie.var_power : float
-        The power of the variance function.
+        The power parameter of the variance function.
 
     See Also
     --------
@@ -1370,9 +1508,9 @@ class Tweedie(Family):
     estimated using the ``estimate_tweedie_power`` function that is part of the
     statsmodels.genmod.generalized_linear_model.GLM class.
     """
-    links = [L.log, L.Power]
+    links = [L.Log, L.Power]
     variance = V.Power(power=1.5)
-    safe_links = [L.log, L.Power]
+    safe_links = [L.Log, L.Power]
 
     def __init__(self, link=None, var_power=1., eql=False):
         self.var_power = var_power
@@ -1381,7 +1519,7 @@ class Tweedie(Family):
             raise ValueError("Tweedie: if EQL=True then var_power must fall "
                              "between 1 and 2")
         if link is None:
-            link = L.log()
+            link = L.Log()
         super(Tweedie, self).__init__(
             link=link, variance=V.Power(power=var_power * 1.))
 
@@ -1480,6 +1618,10 @@ class Tweedie(Family):
 
         References
         ----------
+        R Kaas (2005).  Compound Poisson Distributions and GLM's -- Tweedie's
+        Distribution.
+        https://core.ac.uk/download/pdf/6347266.pdf#page=11
+
         JA Nelder, D Pregibon (1987).  An extended quasi-likelihood function.
         Biometrika 74:2, pp 221-232.  https://www.jstor.org/stable/2336136
         """
@@ -1487,9 +1629,10 @@ class Tweedie(Family):
             # We have not yet implemented the actual likelihood
             return np.nan
 
-        # Equations 9-10 or Nelder and Pregibon
+        # Equations 4 of Kaas
         p = self.var_power
-        llf = np.log(2 * np.pi * scale) + p * np.log(mu) - np.log(var_weights)
+        llf = np.log(2 * np.pi * scale) + p * np.log(endog)
+        llf -= np.log(var_weights)
         llf /= -2
 
         if p == 1:
@@ -1504,9 +1647,8 @@ class Tweedie(Family):
                  - (2 - p) * endog * mu ** (1 - p)
                  + (1 - p) * mu ** (2 - p))
             u *= var_weights / (scale * (1 - p) * (2 - p))
-        llf -= u
 
-        return llf
+        return llf - u
 
     def resid_anscombe(self, endog, mu, var_weights=1., scale=1.):
         r"""
